#ifndef _KINOTIFY_WATCH_H_
#define _KINOTIFY_WATCH_H_

#include "kinotify.h"

#include <QtCore/QObject>


namespace KInotify {

    class WatchPool;

    class Watch : public QObject
    {
	Q_OBJECT

    public:
	~Watch();

	bool isValid() const;
	
	int descriptor() const;

	QString path() const;
	WatchModes mode() const;
	WatchFlags flags() const;
	bool recursive() const;

	void modify( WatchModes mode, WatchFlags flags );

	/**
	 * \return \true if this Watch instance watches changes
	 * in \p path, \p false otherwise.
	 */
//	bool watches( const QString& path );


    signals:
	void watchUserLimitReached();

    private:
	Watch( QObject* parent, WatchPool* pool, 
	       int inotifyId, const QString& path, 
	       WatchModes mode, WatchFlags flags, bool recursive = false );

	/**
	 * Called by sub watches
	 */
	void subWatchRemoved( Watch* subWatch );

	/**
	 * Add a new subwatch
	 */
       	void addSubWatch( const QString& subPath );

	/**
	 * Change the path of this watch (in case it was moved)
	 */
	void updatePath( const QString& path );

	class Private;
	Private* const d;

	friend class DirWatch;
    };
}

#endif
