/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "kio_nepomuk.h"

#include <QtCore/QByteArray>
#include <QtCore/QTextStream>

#include <KComponentData>
#include <KDebug>

#include <nepomuk/resource.h>
#include <nepomuk/resourcemanager.h>
#include <nepomuk/class.h>
#include <nepomuk/property.h>

#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/Statement>
#include <Soprano/StatementIterator>
#include <Soprano/Vocabulary/RDF>


Nepomuk::NepomukProtocol::NepomukProtocol( const QByteArray& poolSocket, const QByteArray& appSocket )
    : KIO::SlaveBase( "nepomuk", poolSocket, appSocket )
{
    ResourceManager::instance()->init();
}


Nepomuk::NepomukProtocol::~NepomukProtocol()
{
}


void Nepomuk::NepomukProtocol::get( const KUrl& url )
{
    kDebug() << url;

    Nepomuk::Resource res( url );

    QByteArray output;

    QString label = res.genericLabel();

    QTextStream os( &output, QIODevice::WriteOnly );
    os << "<html>"
       << "<head><title>" << label << "</title></head>"
       << "<body>";

    os << "<p><b>Warning: this is a premature representation of Nepomuk resources for testing purposes!</b></p>"
       << "<h1>" << label << "</h1>"
       << "<p>Type: " << Types::Class( res.resourceType() ).label() << "</p>";

    os << "<p>Relations:<ul>";

    // query relations
    Soprano::StatementIterator it = ResourceManager::instance()->mainModel()->listStatements( url, Soprano::Node(), Soprano::Node() );
    while ( it.next() ) {
        Soprano::Statement s = it.current();
        if ( s.predicate().uri() != Soprano::Vocabulary::RDF::type() ) {
            Nepomuk::Types::Property p( s.predicate().uri() );
            os << "<li>" << ( p.label().isEmpty() ? p.name() : p.label() ) << ": ";
            if ( s.object().isLiteral() ) {
                os << s.object().toString();
            }
            else {
                os << "<a href=\"" << s.object().uri() << "\">" << Resource( s.object().uri() ).genericLabel() << "</a>";
            }
            os << "</li>";
        }
    }
    os << "</ul>";

    os << "</body></html>";
    os.flush();

    kDebug() << "created data:" << output;

    data( output );
    finished();
}


void Nepomuk::NepomukProtocol::stat( const KUrl& url )
{
    kDebug() << url;

    // for now we do the simple thing
    // FIXME: later, forward tags to the tags kioslave and so on

    KIO::UDSEntry udsEntry;

    // Regular file with rwx permission for all
    udsEntry.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG | S_IRWXU | S_IRWXG | S_IRWXO );

    statEntry( udsEntry );

    finished();
}


void Nepomuk::NepomukProtocol::mimetype( const KUrl& url )
{
    kDebug() << url;

    // for now we do the simple thing
    // FIXME: later, forward tags to the tags kioslave and so on
    mimeType( "text/html" );
    finished();
}


extern "C"
{
    int KDE_EXPORT kdemain( int argc, char **argv ) {

        KComponentData componentData("kio_nepomuk");

        if (argc != 4)
        {
            fprintf(stderr, "Usage: kio_nepomuk protocol domain-socket1 domain-socket2\n");
            exit(-1);
        }

        Nepomuk::NepomukProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();

        return 0;
    }
}
