/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2007-2008 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef _KIO_KMETADATA_TAGS_H_
#define _KIO_KMETADATA_TAGS_H_

#include <kio/forwardingslavebase.h>

class QEventLoop;

namespace Nepomuk {
    class TagsProtocol : public KIO::ForwardingSlaveBase
    {
        Q_OBJECT

    public:
        TagsProtocol( const QByteArray& poolSocket, const QByteArray& appSocket );
        virtual ~TagsProtocol();

        /**
         * List all files and folders tagged with the corresponding tag.
         */
        void listDir( const KUrl& url );

        /**
         * Results in the creation of a new tag.
         */
        void mkdir( const KUrl &url, int permissions );

        /**
         * Will be forwarded for files.
         */
        void get( const KUrl& url );

        /**
         * Not supported.
         */
        void put( const KUrl& url, int permissions, KIO::JobFlags flags );
		
        /**
         * Files and folders can be copied to the virtual folders resulting
         * is assignment of the corresponding tag.
         */
        void copy( const KUrl& src, const KUrl& dest, int permissions, KIO::JobFlags flags );

        /**
         * File renaming will be forwarded.
         * Folder renaming results in renaming of the tag.
         */
        void rename( const KUrl& src, const KUrl& dest, KIO::JobFlags flags );

        /**
         * File deletion means remocing the tag
         * Folder deletion will result in deletion of the tag.
         */
        void del( const KUrl& url, bool isfile );

        /**
         * Files will be forwarded.
         * Folders will be created as virtual folders.
         */
        void mimetype( const KUrl& url );

        /**
         * Files will be forwarded.
         * Folders will be created as virtual folders.
         */
        void stat( const KUrl& url );

    protected:
        /**
         * reimplemented from ForwardingSlaveBase
         */
        bool rewriteUrl( const KUrl& url, KUrl& newURL );

    private Q_SLOTS:
        void slotRedirection( KIO::Job *job, const KUrl& url );
        void slotResult( KJob* job );

    private:
        bool splitUrl( const KUrl& url, QString& tag, QString& path, bool signalError = true );
        void listTags();
        void listTag( const QString& tag );
        KIO::UDSEntry statFile( const KUrl& url );

        QString m_currentTag;
        KUrl m_currentUrl;
        KIO::UDSEntry m_currentUDS;

        QEventLoop m_eventLoop;
    };
}

#endif
