/*
 *
 * $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
 *
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2006-2007 Sebastian Trueg <trueg@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */

#include "mainwindow.h"
#include "../resource-visualization/resourcedelegate.h"
#include "../resource-visualization/abstractresourceguiitem.h"
#include "../search-api/searchcore.h"
#include "../search-api/queryparser.h"
#include "searchhitmodel.h"
#include "kcategorydrawer.h"
#include "kcategorizedview.h"

#include <nepomuk/resourcemanager.h>
#include <nepomuk/resource.h>
#include <nepomuk/class.h>

#include <Soprano/Client/DBusClient>
#include <Soprano/Client/DBusModel>
#include <Soprano/QueryResultIterator>
#include <Soprano/Node>
#include <Soprano/Statement>
#include <Soprano/LiteralValue>
#include <Soprano/StatementIterator>
#include <Soprano/Vocabulary/RDF>
#include <Soprano/Vocabulary/RDFS>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/XMLSchema>

#include <KDebug>
#include <kcategorizedsortfilterproxymodel.h>
#include <KDialog>
#include <KRun>
#include <KComboBox>

#include <QtCore/QCoreApplication>
#include <QtCore/QRegExp>
#include <QtGui/QAbstractItemView>
#include <QtGui/QSortFilterProxyModel>


using namespace Soprano;


MainWindow::MainWindow()
  : QMainWindow(),
    Ui::MainWindow()
{
    setupUi( this );

    Nepomuk::ResourceManager::instance()->init();

    m_searchCore = new Nepomuk::Search::SearchCore( this );
    m_searchCore->setMaxResults( 50 );

    m_searchHitModel = new Nepomuk::SearchHitModel( this );
    m_searchProxyModel = new KCategorizedSortFilterProxyModel( this );
    m_searchProxyModel->setSourceModel( m_searchHitModel );
    m_searchProxyModel->setCategorizedModel( true );
    m_searchProxyModel->sort( 0 );

    m_resultsView->setModel( m_searchProxyModel );
    Nepomuk::ResourceDelegate* delegate = new Nepomuk::ResourceDelegate( m_resultsView );
    delegate->setDisplayMode( Nepomuk::AbstractResourceGuiItem::DisplayFull );
    m_resultsView->setItemDelegateForColumn( 0, delegate );
    m_resultsView->setViewMode( QListView::IconMode );
    m_resultsView->setIconSize( QSize( 48, 48 ) );
    m_resultsView->setCategoryDrawer( new KCategoryDrawer() );
    m_resultsView->setResizeMode( QListView::Adjust );
    m_resultsView->setSpacing( KDialog::spacingHint() );
    m_resultsView->setMouseTracking( true );
    m_resultsView->viewport()->setAttribute( Qt::WA_Hover );

    m_searchCompletion = m_editSearchText->completionObject();

    connect( m_buttonSearch, SIGNAL( clicked() ),
             this, SLOT( slotSearch() ) );
    connect( m_editSearchText, SIGNAL( returnPressed() ),
             this, SLOT( slotSearch() ) );
    connect( m_searchCore, SIGNAL( newResult( const QUrl&, double ) ),
             m_searchHitModel, SLOT( updateResource( const QUrl&, double ) ) );
    connect( m_searchCore, SIGNAL( scoreChanged( const QUrl&, double ) ),
             m_searchHitModel, SLOT( updateResource( const QUrl&, double ) ) );
    connect( m_resultsView, SIGNAL( activated( const QModelIndex& ) ),
             this, SLOT( slotItemActivated( const QModelIndex& ) ) );
}


MainWindow::~MainWindow()
{
    delete m_resultsView->categoryDrawer();
}


void MainWindow::slotSearch()
{
    QString searchText = m_editSearchText->text();

    m_searchHitModel->setResources( QHash<QUrl, double>() );

    if ( !searchText.isEmpty() ) {
        m_searchCompletion->addItem( searchText );

        kDebug() << "Searching for" << searchText;

        Nepomuk::Search::Term term = Nepomuk::Search::QueryParser::parseQuery( searchText );

        kDebug() << "Search term:" << term;

        m_searchCore->query( term );
    }
}


void MainWindow::slotItemActivated( const QModelIndex& index )
{
    QUrl uri = m_searchHitModel->resourceForIndex( m_searchProxyModel->mapToSource( index ) );
    if ( uri.isValid() ) {
        // FIXME: Use a plugin-system like the painting one for execution of resources (maybe integrate it into the gui item)
        new KRun( uri, this );
    }
}

#include "mainwindow.moc"
