/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourcedelegate.h"
#include "abstractresourceguiitem.h"
#include "resourcemodel.h"

#include <QtCore/QModelIndex>
#include <QtCore/QUrl>
#include <QtCore/QHash>
#include <QtCore/QEvent>
#include <QtGui/QAbstractItemView>

#include <KDebug>

Q_DECLARE_METATYPE( Nepomuk::AbstractResourceGuiItem* );

class Nepomuk::ResourceDelegate::Private
{
public:
    Private( ResourceDelegate* parent )
        : displayMode( AbstractResourceGuiItem::DisplayOptimal ),
          q( parent ) {
    }
    ~Private() {
    }

    AbstractResourceGuiItem::DisplayMode displayMode;

private:
    ResourceDelegate* q;
};


Nepomuk::ResourceDelegate::ResourceDelegate( QObject* parent )
    : QAbstractItemDelegate( parent ),
      d( new Private(this) )
{
}


Nepomuk::ResourceDelegate::~ResourceDelegate()
{
    delete d;
}


void Nepomuk::ResourceDelegate::setDisplayMode( AbstractResourceGuiItem::DisplayMode mode )
{
    d->displayMode = mode;
}


Nepomuk::AbstractResourceGuiItem::DisplayMode Nepomuk::ResourceDelegate::displayMode() const
{
    return d->displayMode;
}


QUrl Nepomuk::ResourceDelegate::resource( const QModelIndex& index ) const
{
    return index.data( ResourceModel::ResourceUriRole ).toUrl();
}


void Nepomuk::ResourceDelegate::paint( QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index ) const
{
    AbstractResourceGuiItem* item = index.data( ResourceModel::ResourceGuiItemRole ).value<AbstractResourceGuiItem*>();
    Q_ASSERT( item );
    item->paint( painter, d->displayMode, option, resource( index ) );
}


QSize Nepomuk::ResourceDelegate::sizeHint( const QStyleOptionViewItem& option, const QModelIndex& index ) const
{
    AbstractResourceGuiItem* item = index.data( ResourceModel::ResourceGuiItemRole ).value<AbstractResourceGuiItem*>();
    Q_ASSERT( item );
    return item->sizeHint( d->displayMode, option, resource( index ) );
}


bool Nepomuk::ResourceDelegate::editorEvent( QEvent* event, QAbstractItemModel* model, const QStyleOptionViewItem& option, const QModelIndex& index )
{
    if ( event->type() != QEvent::MouseMove ) {
        AbstractResourceGuiItem* item = index.data( ResourceModel::ResourceGuiItemRole ).value<AbstractResourceGuiItem*>();
        Q_ASSERT( item );
        if ( item->handleEvent( event, d->displayMode, option, resource( index ) ) ) {
            // FIXME:
//            emit dataChanged( index );
            return true;
        }
    }

    return false;
}

#include "resourcedelegate.moc"
