/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "resourcemodel.h"
#include "resourceguiitemfactory.h"
#include "abstractresourceguiitem.h"

#include <QtCore/QUrl>
#include <QtCore/QList>

#include <kdebug.h>

#include <nepomuk/resource.h>

Q_DECLARE_METATYPE( Nepomuk::AbstractResourceGuiItem* );

class Nepomuk::ResourceModel::Private
{
public:
    Private( ResourceModel* parent )
        : q( parent ) {
    }

    // caching the gui items
    ResourceGuiItemFactory factory;
    AbstractResourceGuiItem::DisplayMode displayMode;
    QHash<QString, AbstractResourceGuiItem*> itemCache;

    AbstractResourceGuiItem* itemForResource( const QUrl& url );
    void _k_dataChanged( const QUrl& uri );

private:
    ResourceModel* q;
};


Nepomuk::AbstractResourceGuiItem* Nepomuk::ResourceModel::Private::itemForResource( const QUrl& url )
{
    Q_ASSERT( url.isValid() );
    QHash<QString, AbstractResourceGuiItem*>::const_iterator it = itemCache.find( url.toString() );
    if ( it != itemCache.end() ) {
        return it.value();
    }
    else {
        AbstractResourceGuiItem* item = factory.guiItemForResource( url );
        connect( item, SIGNAL( dataChanged( const QUrl& ) ),
                 q, SLOT( _k_dataChanged( const QUrl& ) ) );
        itemCache.insert( url.toString(), item );
        return item;
    }
}


void Nepomuk::ResourceModel::Private::_k_dataChanged( const QUrl& uri )
{
    kDebug() << uri;
    QModelIndex index = q->indexForResource( uri );
    Q_ASSERT( index.isValid() );
    emit q->dataChanged( index, index );
}



Nepomuk::ResourceModel::ResourceModel( QObject* parent )
    : QAbstractItemModel( parent ),
      d( new Private(this) )
{
}


Nepomuk::ResourceModel::~ResourceModel()
{
    delete d;
}


int Nepomuk::ResourceModel::columnCount( const QModelIndex& ) const
{
    return 1;
}


QVariant Nepomuk::ResourceModel::data( const QModelIndex& index, int role ) const
{
    QUrl uri = resourceForIndex( index );
    Q_ASSERT( uri.isValid() );

    switch( role ) {
    case ResourceGuiItemRole:
        return qVariantFromValue( d->itemForResource( uri ) );
    case ResourceUriRole:
        return uri;
    default:
        return d->itemForResource( uri )->data( uri, role );
    }
}


QModelIndex Nepomuk::ResourceModel::parent( const QModelIndex& ) const
{
    return QModelIndex();
}

#include "resourcemodel.moc"
