/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#ifndef _NEPOMUK_RESOUCE_MODEL_H_
#define _NEPOMUK_RESOUCE_MODEL_H_

#include <QtCore/QAbstractItemModel>

#include <nepomuk/nepomuk_export.h>

namespace Nepomuk {
    class NEPOMUK_EXPORT ResourceModel : public QAbstractItemModel
    {
        Q_OBJECT

    public:
        ResourceModel( QObject* parent = 0 );
        ~ResourceModel();

        /**
         * Some custom roles that are used within the resource visualization framework.
         * \sa data.
         */
        enum ResourceRoles {
            /**
             * The gui item itself as a pointer to AbstractResourceGuiItem. 
             * This is used in the ResourceDelegate to retrieve the actual 
             * item that will do the painting.
             */
            ResourceGuiItemRole = 1000,

            /**
             * The URI of the resource.
             */
            ResourceUriRole
        };

        int columnCount( const QModelIndex& parent = QModelIndex() ) const;
        virtual QUrl resourceForIndex( const QModelIndex& index ) const = 0;
        virtual QModelIndex indexForResource( const QUrl& uri ) const = 0;
        virtual QVariant data( const QModelIndex& index, int role = Qt::DisplayRole ) const;

        QModelIndex parent( const QModelIndex& index ) const;

    private:
        class Private;
        Private* const d;

        Q_PRIVATE_SLOT( d, void _k_dataChanged( const QUrl& ) )
    };
}

#endif
