/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#ifndef _NEPOMUK_SEARCH_THREAD_H_
#define _NEPOMUK_SEARCH_THREAD_H_

#include <QtCore/QThread>
#include <QtCore/QHash>
#include <QtCore/QUrl>

#include "term.h"


uint qHash( const QUrl& );

namespace Soprano {
    class Model;
    namespace Client {
        class DBusClient;
    }
}

namespace Nepomuk {
    namespace Search {
        class SearchThread : public QThread
        {
            Q_OBJECT

        public:
            SearchThread( QObject* parent = 0 );
            ~SearchThread();

            /**
             * Use instead of QThread::start()
             */
            void query( const Term& term, double cutOffScore, int maxResults );
            void cancel();

            int maxResults() const { return m_maxResults; }
            double cutOffScore() const { return m_cutOffScore; }

        signals:
            void newResult( const QUrl& resource, double score );

        protected:
            void run();

        private:
            QList<QUrl> matchFieldName( const QString& field );
            QHash<QUrl, double> search( const Term& term, double baseScore, bool reportResults = false );
            QHash<QUrl, double> relationSearch( const QUrl& resource, double baseScore, bool reportResults = false );
            QHash<QUrl, double> equalsFieldSearch( const Term& query, double baseScore, bool reportResults = false );
            QHash<QUrl, double> containsFieldSearch( const Term& query, double baseScore, bool reportResults = false );
            QHash<QUrl, double> plainLiteralSearch( const QString& query, double baseScore, bool reportResults = false );
            QHash<QUrl, double> literalSearch( const Term& query, double baseScore, bool reportResults = false );
            QHash<QUrl, double> andSearch( const QList<Term>& terms, double baseScore, bool reportResults = false );
            QHash<QUrl, double> orSearch( const QList<Term>& terms, double baseScore, bool reportResults = false );

            Soprano::Client::DBusClient* m_client;
            Soprano::Model* m_model;

            Term m_searchTerm;
            double m_cutOffScore;
            int m_maxResults;

            // status
            int m_numResults;
            bool m_canceled;
        };
    }
}

#endif
